package org.iphsoft.simon1;

import android.app.Dialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.os.Build;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.view.Gravity;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.view.ViewParent;
import android.view.ViewTreeObserver.OnGlobalLayoutListener;
import android.view.WindowManager;
import android.view.animation.AnimationUtils;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.google.android.gms.auth.api.Auth;
import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.auth.api.signin.GoogleSignInResult;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesClient;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.R;

import org.iphsoft.simon1.AdHelper.AdInfo;
import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.stats.StatisticsManager;
import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.ui.GameMenuUiHelper;
import org.iphsoft.simon1.ui.VideoActivity;
import org.iphsoft.simon1.util.BackgroundMusicPlayer;
import org.iphsoft.simon1.util.BitmapUtils;
import org.iphsoft.simon1.util.DialogUtils;
import org.iphsoft.simon1.util.FileUtils;
import org.iphsoft.simon1.util.IHNMUtils;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.IntentUtils;
import org.iphsoft.simon1.util.MediaPlayerHelper;
import org.iphsoft.simon1.util.MyLog;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class GameMenuActivity extends PreferenceActivity {

	public static final String EXTRA_MID_GAME = "GameMenuActivity.EXTRA_MID_GAME";
	public static final String EXTRA_FROM_POSTCARD = "GameMenuActivity.EXTRA_FROM_POSTCARD";
	public static final String EXTRA_SAVE_SLOT = "GameMenuActivity.EXTRA_SAVE_SLOT";

	public static final int RESULT_NONE = 0;
	public static final int RESULT_REQUEST_RESTART = 1;
	public static final int RESULT_REQUEST_SAVE = 2;
	public static final int RESULT_REQUEST_LOAD = 3;
	public static final int RESULT_REQUEST_RESTART_LOAD = 4;
	public static final int RESULT_REQUEST_QUIT = 5;
	public static final int RESULT_REQUEST_OPEN_HOUSE = 6;

	private static final int GOOGLE_GAMES_RC_SIGN_IN  = 1003;
	private final static int GOOGLE_GAMES_RESOLUTION_REQUEST = 9000;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		MyLog.d("GameMenuActivity: onCreate: ");

		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);

		Intent launchingIntent = getIntent();
		mMidGame = launchingIntent.getBooleanExtra(EXTRA_MID_GAME, false);

		// HACK for quieting down a crash when changing locale
		if (savedInstanceState != null && mMidGame)
		{
			Serializable oldLocaleObject = savedInstanceState.getSerializable("locale");
			if (oldLocaleObject != null)
			{
				Locale oldLocale = (Locale) oldLocaleObject;
				Locale currentLocale = getResources().getConfiguration().locale;
				if (!oldLocale.equals(currentLocale))
				{
					MyLog.w("GameMenuActivity: onCreate: locale change in game activity");
					finish();
					ActivityAnimationUtil.makeActivityNullTransition(this);
					return;
				}
			}
		}

		mImmersiveModeHelper = new ImmersiveModeHelper(this);

		// Set hardware acceleration to make alpha transitions fluid
		if (Build.VERSION.SDK_INT >= 11)
		{
			getWindow().setFlags(WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED,
					WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED);
		}

		setContentView(setupContentView());

		MyLog.d("GameMenuActivity: onCreate: display metrics: density "
				+ getResources().getDisplayMetrics().densityDpi
				+ ", screen size type "
				+ (getResources().getConfiguration().screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK));

		switchStateMain();

		// Set preference defaults if needed
		AndroidPortAdditions.instance().setPreferenceDefaults();

		// Initialize the preferences from XML
		addPreferencesFromResource(R.xml.preferences);

		// WORKAROUND: for Simon 1 German version - make sure we never use
		// "voice and subtitles"
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1)
		{
			AndroidPortAdditions.instance().germanVersionForceVoiceOnly(getPreferenceManager());
		}

		if (AndroidPortAdditions.USE_SHADER_FILES)
		{
			// TESTING: copy shader and video to ext. storage
			FileUtils.copyAssetToExternalStorage(this, "fragment.glsl", null);
			FileUtils.copyAssetToExternalStorage(this, "vertex.glsl", null);
			FileUtils.copyAssetToExternalStorage(this, "lq_fragment.glsl", null);
			FileUtils.copyAssetToExternalStorage(this, "lq_vertex.glsl", null);

			if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
			{
				FileUtils.copyAssetToExternalStorage(this, "rect_highlighting_vertex.glsl", null);
				FileUtils.copyAssetToExternalStorage(this, "rect_highlighting_fragment.glsl", null);
			}
		}

		// Set the parent directory for storing the game files, and also check
		// USB storage availability
		// (This is the entry point activity)
		if (!mMidGame)
		{
			File externalDir = getExternalFilesDir(null);
			if (externalDir != null)
			{
				AndroidPortAdditions.instance().setGameFileParentDir(externalDir.getAbsolutePath());
			} else
			{
				MyLog.e("GameMenuActivity: onCreate: no external storage");

				// Show a dialog
				Dialog dlg = DialogUtils.createNegativePositiveDialog(this, 0, R.string.ok,
						R.string.storage_unavailable, null, null);
				dlg.setOnDismissListener(new OnDismissListener()
				{

					@Override
					public void onDismiss(DialogInterface arg0)
					{
						// Finish
						GameMenuActivity.this.finish();
					}
				});

				dlg.show();

				return;
			}
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G && checkGooglePlayServices()) {
			googleGamesSignInSilently();
		}

		// Play T7G voice parts
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G && mMidGame == false)
		{
			if (AndroidPortAdditions.instance().getSaveSlotsIndexes().size() == 0)
			{
				// "Welcome to my house"
				MediaPlayerHelper.playAudioResource(this, R.raw.welcome_to_my_house);
			} else
			{
				// "Back for more?"
				MediaPlayerHelper.playAudioResource(this, R.raw.back_for_more);
			}
		}

		if (AndroidPortAdditions.GAME_TYPE != GameType.IHNM)
		{
			AdHelper.startBackgroundAdCheck();
		}
	}

	@Override
	public void onSaveInstanceState(Bundle outState)
	{
		MyLog.d("GameMenuActivity: onSaveInstanceState: ");

		Locale currentLocale = getResources().getConfiguration().locale;
		outState.putSerializable("locale", currentLocale);
		super.onSaveInstanceState(outState);
	}

	@Override
	protected void onStart()
	{
		super.onStart();

		//StatisticsManager.getStatistics().reportActivityStart(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC && !mMidGame)
		{
			BackgroundMusicPlayer.addBackgroundMusicActivity();
		}
	}

	@Override
	protected void onStop()
	{
		super.onStop();

		//StatisticsManager.getStatistics().reportActivityStop(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC && !mMidGame)
		{
			BackgroundMusicPlayer.removeBackgroundMusicActivity();
		}
	}

	@Override
	protected void onResume()
	{
		// TODO Auto-generated method stub
		super.onResume();

		mImmersiveModeHelper.supportFullScreenImmersiveMode();

	}

	private void showTutorialPromptDialog()
	{
		DialogUtils.showNegativePositiveDialogOrActivity(this, R.string.no, R.string.yes,
				R.string.tutorial_prompt, new DialogInterface.OnClickListener()
				{

					@Override
					public void onClick(DialogInterface arg0, int arg1)
					{

						// If pressed "no", start new game
						startActivity(new Intent(GameMenuActivity.this,
								PrepareGameFilesActivity.class).putExtra(EXTRA_MID_GAME, mMidGame));
						finish();
						ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);

					}
				}, new DialogInterface.OnClickListener()
				{

					@Override
					public void onClick(DialogInterface arg0, int arg1)
					{

						// If pressed "yes", show tutorial
						startActivity(new Intent(GameMenuActivity.this, VideoActivity.class)
								.putExtra(VideoActivity.EXTRA_VIDEO_RESOURCE, R.raw.tutorial));
						ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
					}
				}, mMidGame);

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			ActivityAnimationUtil.makeActivityNullTransition(this);
		}

	}

	private void showLoadDialog()
	{
		// Generate a list of save slots to display
		List<String> allSaveSlots = AndroidPortAdditions.instance().getSaveSlots();
		mSaveSlotIndexes = AndroidPortAdditions.instance().getSaveSlotsIndexes();

		List<String> displayedSaveSlots = new ArrayList<String>();

		for (Integer i : mSaveSlotIndexes)
		{
			displayedSaveSlots.add(allSaveSlots.get(i));
		}

		int negText = AndroidPortAdditions.GAME_TYPE == GameType.IHNM ? R.string.back : R.string.cancel;
		DialogUtils.showListDialog(this, negText, R.string.load_game, R.string.choose_load_slot,
				new DialogInterface.OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dlg, int which)
					{
						// Convert index to save slot
						int slot = mSaveSlotIndexes.get(which) + 1;

						requestLoadWithRestart(slot);
					}
				}, displayedSaveSlots.toArray(new String[0]), mMidGame);
	}

	private void requestLoadWithRestart(int slot)
	{
		MyLog.d("GameMenuActivity: requestLoadWithRestart: " + slot);

		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			requestLoadWithRestartIHNM(slot);
		} else
		{
			requestLoadWithRestartNormal(slot);
		}
	}

	private void requestLoadWithRestartNormal(int slot)
	{
		if (mMidGame)
		{
			// Request the game activity to restart and load
			Intent data = new Intent().putExtra(EXTRA_SAVE_SLOT, slot);

			setResult(RESULT_REQUEST_RESTART_LOAD, data);
		} else
		{
			// Start the game activity with a load slot
			Intent loadIntent = new Intent(this, PrepareGameFilesActivity.class).putExtra(
					EXTRA_MID_GAME, mMidGame);
			loadIntent.putExtra(ScummVMActivity.EXTRA_STARTING_SAVE_SLOT, slot);
			startActivity(loadIntent);
		}

		finish();
		if (AndroidPortAdditions.GAME_TYPE != GameType.IHNM)
			ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
		else
			ActivityAnimationUtil.makeActivityNullTransition(GameMenuActivity.this);
	}

	private void requestLoadWithRestartIHNM(final int slot)
	{
		String languageSetting = PreferenceManager.getDefaultSharedPreferences(this).getString(
				"pref_language", null);
		final String saveFileLanguage = IHNMUtils.getLanguageSettingForSaveSlot(GameMenuActivity.this, slot, null);

		// Check if the current save file corresponds to the current language
		if (languageSetting.equals(saveFileLanguage))
		{
			requestLoadWithRestartNormal(slot);
			return;
		}

		// Otherwise, offer to switch the language
		DialogUtils.createNegativePositiveDialog(this, R.string.no, R.string.yes,
				R.string.load_change_language_popup, null, new DialogInterface.OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dialog, int which)
					{
						// Change the language setting
						PreferenceManager.getDefaultSharedPreferences(GameMenuActivity.this).edit()
								.putString("pref_language", saveFileLanguage).commit();
						AndroidPortAdditions.instance().setGameFilesSize(-1);

						// Load
						requestLoadWithRestartNormal(slot);
					}
				}).show();
	}

	private View setupContentView()
	{
		View contentView = getLayoutInflater().inflate(R.layout.game_menu, null);

		mMenuFrame = (FrameLayout) contentView.findViewById(R.id.menu_frame);
		mBgFrame = contentView.findViewById(R.id.bg_frame);
		mImgLogo = (ImageView) contentView.findViewById(R.id.img_logo);
		mTxtMenuTitle = (TextView)contentView.findViewById(R.id.txt_menu_title);

		mSubTitleView = contentView.findViewById(R.id.subtitle_view);
		mSphinxView = contentView.findViewById(R.id.img_sphinx);
		mMenuBottomMarginView = contentView.findViewById(R.id.menuBottomMarginView);
		mMenuTopMarginView = contentView.findViewById(R.id.menuTopMarginView);

		mShareBtn = contentView.findViewById(R.id.btnShare);
		mShareBtn.setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View v)
			{
				shareApp();
			}
		});

		View aboutBtn = contentView.findViewById(R.id.btnAbout);
		if (aboutBtn != null)
		{
			aboutBtn.setOnClickListener(new OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					startActivity(new Intent(GameMenuActivity.this, AboutActivity.class).putExtra(
							EXTRA_MID_GAME, mMidGame));
					ActivityAnimationUtil.makeActivityNullTransition(GameMenuActivity.this);
				}
			});
		}

		// T7G - Check for open house mode
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G
				&& AndroidPortAdditions.instance().getOpenHouseFlag())
		{
			MyLog.d("GameMenuActivity: setupContentView: open house mode");

			// Setup the board corners behavior
			OnClickListener boardCornerListener = new OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					if (mMidGame)
					{
						showOpenHouseDialog();
					} else
					{
						startOpenHouse();
					}
				}
			};

			contentView.findViewById(R.id.openHouseCorner1).setOnClickListener(boardCornerListener);
			contentView.findViewById(R.id.openHouseCorner2).setOnClickListener(boardCornerListener);
			contentView.findViewById(R.id.openHouseCorner3).setOnClickListener(boardCornerListener);
			contentView.findViewById(R.id.openHouseCorner4).setOnClickListener(boardCornerListener);
		}

		mCurrentContentView = contentView;

		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
				|| AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ
				|| AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			// Set the constant menu background
			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.bg);
		} else if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			// Set the constant menu background
			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.splash);

			// Set the 2nd background
			View bg2 = contentView.findViewById(R.id.bg2);
			if (AndroidPortAdditions.instance().getOpenHouseFlag())
				BitmapUtils.setScaledBackground(this, bg2, R.drawable.bg_openhouse);
			else
				BitmapUtils.setScaledBackground(this, bg2, R.drawable.bg);

		}

		return contentView;
	}

	private void switchStateMain()
	{
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			/*
			 * mBgSwitcher.setInAnimation(null);
			 * mBgSwitcher.setOutAnimation(null);
			 * mLogoSwitcher.setInAnimation(null);
			 * mLogoSwitcher.setOutAnimation(null);
			 */

			// Set the background if we just got into the menu
			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.bg);

			mImgLogo.setImageResource(R.drawable.bg_logo);

			mMenuBottomMarginView.setLayoutParams(new LinearLayout.LayoutParams(
					LayoutParams.MATCH_PARENT, 0, SIMON2_MAIN_MENU_BOTTOM_MARGIN_WEIGHT));
			mMenuTopMarginView.setLayoutParams(new LinearLayout.LayoutParams(
					LayoutParams.MATCH_PARENT, 0, SIMON2_MAIN_MENU_TOP_MARGIN_WEIGHT));
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			// Set the background if we just got into the menu
			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.bg);

			findViewById(R.id.btnBack).setVisibility(View.GONE);
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			mSphinxView.setVisibility(View.VISIBLE);
		}

		mMenuState = MenuState.MAIN;

		// Inflate the main layout
		mMenuFrame.removeAllViews();
		mSubTitleView.setVisibility(View.VISIBLE);
		
		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			mTxtMenuTitle.setText("");
		}

		if (mMainMenuView == null)
		{
			mMainMenuView = View.inflate(this, R.layout.main_menu_frame, null);
			mMainMenuView.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
					LayoutParams.FILL_PARENT));
		}

		// Detach the cached main menu from its parent
		ViewParent mainMenuParent = mMainMenuView.getParent();
		if (mainMenuParent != null && mainMenuParent instanceof ViewGroup)
		{
			((ViewGroup) mainMenuParent).removeAllViews();
		}

		mMenuFrame.addView(mMainMenuView);

		// Setup continue button
		View btnContinue = findViewById(R.id.btnContinue);
		btnContinue.setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{

				if (mMidGame)
				{
					// Just finish the menu and go back to game
					setResult(RESULT_NONE);
					finish();
					ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);

				} else
				{
					// Load the last save
					requestLoadWithRestart(AndroidPortAdditions.instance().getLastSaveSlot());
				}

			}
		});

		// Disallow continue if there is no last save and game is not in
		// progress
		if (!mMidGame && AndroidPortAdditions.instance().getLastSaveSlot() == -1)
		{
			btnContinue.setEnabled(false);
		}

		// Setup new game button
		findViewById(R.id.btnNewGame).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				AndroidPortAdditions.instance().setStartNewGameAltIntro(false);
				onNewGameClick();
			}
		});

		if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
		{
			// FOTAQ: Setup new game button long click
			findViewById(R.id.btnNewGame).setOnLongClickListener(new OnLongClickListener()
			{

				@Override
				public boolean onLongClick(View v)
				{
					AndroidPortAdditions.instance().setStartNewGameAltIntro(true);
					onNewGameClick();
					return true;
				}
			});
		}

		// Setup save button
		View btnSave = findViewById(R.id.btnSave);
		btnSave.setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				Intent data = new Intent().putExtra(EXTRA_FROM_POSTCARD, getIntent()
						.getBooleanExtra(EXTRA_FROM_POSTCARD, false));
				setResult(RESULT_REQUEST_SAVE, data);
				finish();
				ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);

			}
		});
		btnSave.setEnabled(mMidGame);

		// Setup load button
		View btnLoad = findViewById(R.id.btnLoad);
		btnLoad.setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{

				showLoadDialog();
			}
		});

		// Disable load button if there are no saves
		if (AndroidPortAdditions.instance().getSaveSlotsIndexes().size() == 0)
		{
			btnLoad.setEnabled(false);
		}

		// Setup quit button
		final View btnExit = findViewById(R.id.btnExit);
		btnExit.setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				if (mMidGame)
				{
					setResult(RESULT_REQUEST_QUIT);
				} else if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
				{
					// "Come back!"
					MediaPlayerHelper.playAudioResource(GameMenuActivity.this, R.raw.come_back);
				}

				finish();
				ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);

			}
		});

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G && checkGooglePlayServices()) {
			// Setup Google sign in button
			final View googleSignInBtn = findViewById(R.id.btnGoogleSignIn);
			googleSignInBtn.setOnClickListener(new OnClickListener() {

				@Override
				public void onClick(View arg0) {
					// start the asynchronous sign in flow
					googleGamesStartSignInIntent();
				}

			});
		}

		// Setup tutorial button
		if (AndroidPortAdditions.HAS_TUTORIAL)
		{
			findViewById(R.id.btnTutorial).setOnClickListener(new OnClickListener()
			{

				@Override
				public void onClick(View arg0)
				{

					AndroidPortAdditions.instance().setTutorialOffered(true);

					startActivity(new Intent(GameMenuActivity.this, VideoActivity.class).putExtra(
							VideoActivity.EXTRA_VIDEO_RESOURCE, R.raw.tutorial));
					ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
				}
			});
		}

		if (AndroidPortAdditions.HAS_EXTRAS)
		{

			// Setup extras button
			findViewById(R.id.btnExtras).setOnClickListener(new OnClickListener()
			{

				@Override
				public void onClick(View arg0)
				{
					startActivity(new Intent(GameMenuActivity.this, PrepareGameFilesActivity.class)
							.putExtra(PrepareGameFilesActivity.EXTRA_SHOW_EXTRAS_SCREEN, true));
					ActivityAnimationUtil.makeActivityNullTransition(GameMenuActivity.this);
				}
			});
		}

		// Setup options button
		findViewById(R.id.btnOptions).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				// Save the current settings before changes
				SharedPreferences prefs = PreferenceManager
						.getDefaultSharedPreferences(GameMenuActivity.this);
				mPreviousMusicSetting = prefs.getString("pref_music_mode", null);
				mPreviousVoiceSetting = prefs.getString("pref_voice_mode", null);
				mPreviousLanguageSetting = prefs.getString("pref_language", null);
				mPreviousScalerSetting = prefs.getString("pref_scaler_option", null);
				mPreviousMusicVolSetting = prefs.getInt("pref_music_volume", 192);
				mPreviousSpeechVolSetting = prefs.getInt("pref_speech_volume", 192);
				mPreviousSfxVolSetting = prefs.getInt("pref_sfx_volume", 192);
				mPreviousSubtitleSpeedSetting = prefs.getInt("pref_subtitle_speed", 60);

				switchStateOptions();
			}
		});

		// Setup ad button
		final ImageView btnAd = (ImageView) findViewById(R.id.btnAd);

		// Get the ad information and adjust the button accordingly
		AdInfo adInfo = null;
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G || AndroidPortAdditions.GAME_TYPE == GameType.SIMON1) { // Force remove ad button for Google Play Pass (NBO) titles
			findViewById(R.id.btnAd).setVisibility(View.GONE);
		}
		else {
			findViewById(R.id.btnAd).setVisibility(View.GONE); // Force remove ad button anyway due to Google finding it a violation of their Ad Content policy
			//adInfo = AdHelper.getAdToDisplay();
		}
		if (adInfo != null)
		{
			if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
			{
				findViewById(R.id.adFrame).setVisibility(View.VISIBLE);

				ImageView adIcon = (ImageView) findViewById(R.id.adIcon);
				adIcon.setImageResource(adInfo.adIconResourceId);

				findViewById(R.id.extraButtonPaddingNormal).setVisibility(View.VISIBLE);
				findViewById(R.id.extraButtonPadding1).setVisibility(View.GONE);
				findViewById(R.id.extraButtonPadding2).setVisibility(View.GONE);

			} else
			{
				btnAd.setImageResource(adInfo.adIconResourceId);
				if (AndroidPortAdditions.GAME_TYPE == GameType.T7G && checkGooglePlayServices()) {
					if (findViewById(R.id.btnGoogleSignIn).getVisibility() == View.VISIBLE && findViewById(R.id.btnAd) != null) {
						findViewById(R.id.btnAd).setVisibility(View.GONE);
					}
				}
			}

			btnAd.setOnClickListener(new OnClickListener()
			{

				@Override
				public void onClick(View arg0)
				{
					IntentUtils.startAppStoreDeveloperActivity(GameMenuActivity.this);
				}
			});

			// Adjust the ad button to be the same height as other buttons
			mMenuFrame.getViewTreeObserver().addOnGlobalLayoutListener(new OnGlobalLayoutListener()
			{

				@Override
				public void onGlobalLayout()
				{
					MyLog.d("GameMenuActivity.switchStateMain().new OnGlobalLayoutListener() {...}: onGlobalLayout: height "
							+ btnExit.getHeight());

					if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
							|| AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
					{
						btnAd.setAdjustViewBounds(true);
						btnAd.setMaxHeight(btnExit.getHeight());
					}

					mMenuFrame.getViewTreeObserver().removeGlobalOnLayoutListener(this);
				}
			});
		} else
		{
			// No ad to display - remove the button
			if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
					|| AndroidPortAdditions.GAME_TYPE == GameType.T7G)
				btnAd.setVisibility(View.GONE);
			else if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
				btnAd.setVisibility(View.INVISIBLE);
			else if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
			{
				findViewById(R.id.adFrame).setVisibility(View.GONE);

				findViewById(R.id.extraButtonPaddingNormal).setVisibility(View.GONE);
				findViewById(R.id.extraButtonPadding1).setVisibility(View.VISIBLE);
				findViewById(R.id.extraButtonPadding2).setVisibility(View.VISIBLE);
			}
		}
	}

	private void onNewGameClick()
	{
		if (mMidGame)
		{
			// Show confirmation dialog for new game
			DialogUtils.showNegativePositiveDialogOrActivity(GameMenuActivity.this, R.string.no,
					R.string.yes, R.string.new_game_confirmation, null,
					new DialogInterface.OnClickListener()
					{

						@Override
						public void onClick(DialogInterface arg0, int arg1)
						{
							// Start a new game
							setResult(RESULT_REQUEST_RESTART);
							finish();
							ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
						}
					}, mMidGame);

			if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
			{
				ActivityAnimationUtil.makeActivityNullTransition(GameMenuActivity.this);
			}

		} else
		{
			// Show tutorial prompt on first time, if tutorial wasn't
			// shown
			if (!AndroidPortAdditions.instance().wasTutorialOffered()
					&& AndroidPortAdditions.HAS_TUTORIAL)
			{
				AndroidPortAdditions.instance().setTutorialOffered(true);
				showTutorialPromptDialog();
				return;
			}

			startActivity(new Intent(GameMenuActivity.this, PrepareGameFilesActivity.class)
					.putExtra(EXTRA_MID_GAME, mMidGame));
			finish();
			ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
		}
	}

	private void switchStateOptions()
	{
		View oldContentView = null, newContentView = null;
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			oldContentView = mCurrentContentView;
			newContentView = setupContentView();

			// Set the background
			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.bg);

			mImgLogo.setImageResource(R.drawable.bg_logo);

			mMenuBottomMarginView.setLayoutParams(new LinearLayout.LayoutParams(
					LayoutParams.MATCH_PARENT, 0, SIMON2_MAIN_MENU_BOTTOM_MARGIN_WEIGHT));
			mMenuTopMarginView.setLayoutParams(new LinearLayout.LayoutParams(
					LayoutParams.MATCH_PARENT, 0, SIMON2_MAIN_MENU_TOP_MARGIN_WEIGHT));
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			oldContentView = mCurrentContentView;
			newContentView = setupContentView();

			// Set the background
			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.bg_settings);

			findViewById(R.id.btnBack).setVisibility(View.VISIBLE);
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			mSphinxView.setVisibility(View.VISIBLE);
		}

		// Show the subtitle view again
		mSubTitleView.setVisibility(View.VISIBLE);
		
		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			mTxtMenuTitle.setText(R.string.settings);
		}

		// Inflate the main layout
		mMenuFrame.removeAllViews();

		View.inflate(this, R.layout.settings_frame, mMenuFrame);

		// Animate if needed
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			if (mMenuState == MenuState.OPTIONS_LEVEL2)
			{
				// Animate the new content view
				newContentView.startAnimation(AnimationUtils.loadAnimation(this,
						R.anim.activity_fade_in));
				oldContentView.startAnimation(AnimationUtils.loadAnimation(this,
						R.anim.activity_fade_out));
			}

			setContentView(newContentView);
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			setContentView(newContentView);
		}

		mMenuState = MenuState.OPTIONS_LEVEL1;

		findViewById(R.id.btnBack).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				backMenuState();
			}
		});

		findViewById(R.id.btnMusic).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				switchStatePreference(getPreferenceManager().findPreference("pref_music_settings"));
			}
		});

		findViewById(R.id.btnVoice).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				switchStatePreference(getPreferenceManager().findPreference("pref_voice_settings"));
			}
		});

		findViewById(R.id.btnLanguage).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				switchStatePreference(getPreferenceManager().findPreference("pref_language"));
			}
		});

		findViewById(R.id.btnGraphics).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				switchStatePreference(getPreferenceManager().findPreference("pref_scaler_option"));
			}
		});

		findViewById(R.id.btnControl).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				switchStatePreference(getPreferenceManager().findPreference("pref_control_mode"));
			}
		});

		View btnAbout = findViewById(R.id.btnAbout);
		if (btnAbout != null)
		{
			btnAbout.setOnClickListener(new OnClickListener()
			{

				@Override
				public void onClick(View arg0)
				{
					startActivity(new Intent(GameMenuActivity.this, AboutActivity.class).putExtra(
							EXTRA_MID_GAME, mMidGame));
					if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
							|| AndroidPortAdditions.GAME_TYPE == GameType.T7G
							|| AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
					{
						ActivityAnimationUtil.makeActivityNullTransition(GameMenuActivity.this);
					} else if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
					{
						ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
					}
				}
			});
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G && checkGooglePlayServices()) {
			View btnAchievements = findViewById(R.id.btnAchievements);
			if (btnAchievements != null) {
				btnAchievements.setOnClickListener(new OnClickListener() {

					@Override
					public void onClick(View arg0) {
						showAchievements();
						// deprecated
						//startActivity(new Intent(GameMenuActivity.this, FacebookLoginActivity.class));

						ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
					}
				});
			}
		}
	}

	private static final int RC_ACHIEVEMENT_UI = 9003;

	private void showAchievements() {
		if (GoogleSignIn.getLastSignedInAccount(this) != null) {
			Games.getAchievementsClient(this, GoogleSignIn.getLastSignedInAccount(this))
					.getAchievementsIntent()
					.addOnSuccessListener(new OnSuccessListener<Intent>() {
						@Override
						public void onSuccess(Intent intent) {
							startActivityForResult(intent, RC_ACHIEVEMENT_UI);
						}
					});
		}
		else {
			googleGamesStartSignInIntent();
		}
	}

	private boolean checkGooglePlayServices() {
		if (AndroidPortAdditions.APP_STORE == AndroidPortAdditions.AppStore.GOOGLE_PLAY) {
			GoogleApiAvailability googleAPI = GoogleApiAvailability.getInstance();
			int result = googleAPI.isGooglePlayServicesAvailable(this);
			if (result != ConnectionResult.SUCCESS) {
				if (googleAPI.isUserResolvableError(result)) {
					googleAPI.getErrorDialog(this, result,
							GOOGLE_GAMES_RESOLUTION_REQUEST).show();
				}
				return false;
			}
			return true;
		}
		else {
			return false;
		}
	}

	private void googleGamesSignInSilently() {
		GoogleSignInOptions signInOptions = GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN;
		GoogleSignInAccount account = GoogleSignIn.getLastSignedInAccount(this);
		if (GoogleSignIn.hasPermissions(account, signInOptions.getScopeArray())) {
			// Already signed in.
			// The signed in account is stored in the 'account' variable.
			GoogleSignInAccount googleSignInAccount = account;
			GamesClient gamesClient = Games.getGamesClient(GameMenuActivity.this, googleSignInAccount);
			gamesClient.setViewForPopups(findViewById(android.R.id.content));
			gamesClient.setGravityForPopups(Gravity.TOP | Gravity.CENTER_HORIZONTAL);

			// Hide the sign-in button
			findViewById(R.id.btnGoogleSignIn).setVisibility(View.GONE);
			if (findViewById(R.id.btnAd) != null) {
				findViewById(R.id.btnAd).setVisibility(View.VISIBLE);
			}
		} else {
			// Haven't been signed-in before. Try the silent sign-in first.
			GoogleSignInClient signInClient = GoogleSignIn.getClient(this, signInOptions);
			signInClient
					.silentSignIn()
					.addOnCompleteListener(
							this,
							new OnCompleteListener<GoogleSignInAccount>() {
								@Override
								public void onComplete(@NonNull Task<GoogleSignInAccount> task) {
									if (task.isSuccessful()) {
										// The signed in account is stored in the task's result.
										GoogleSignInAccount googleSignInAccount = task.getResult();
										GamesClient gamesClient = Games.getGamesClient(GameMenuActivity.this, googleSignInAccount);
										gamesClient.setViewForPopups(findViewById(android.R.id.content));
										gamesClient.setGravityForPopups(Gravity.TOP | Gravity.CENTER_HORIZONTAL);

										// Hide the sign-in button
										findViewById(R.id.btnGoogleSignIn).setVisibility(View.GONE);
										if (findViewById(R.id.btnAd) != null) {
											findViewById(R.id.btnAd).setVisibility(View.VISIBLE);
										}
									} else {
										SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(GameMenuActivity.this);
										mFirstGoogleSignIn = sharedPref.getBoolean("FirstGoogleSignIn", true);
										// By Google's guidelines: player will need to sign-in explicitly using via UI.
										if (mFirstGoogleSignIn) {
											googleGamesStartSignInIntent();
										}
										else {
											// Show the sign-in button
											findViewById(R.id.btnGoogleSignIn).setVisibility(View.VISIBLE);
											if (findViewById(R.id.btnAd) != null) {
												findViewById(R.id.btnAd).setVisibility(View.GONE);
											}
										}
									}
								}
							});
		}
	}

	private void googleGamesStartSignInIntent() {  // NOTE: this can be tested only in "Release" builds
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(this);
		SharedPreferences.Editor editor = sharedPref.edit();
		editor.putBoolean("FirstGoogleSignIn", false);
		editor.commit();
		mFirstGoogleSignIn = false;
		GoogleSignInClient signInClient = GoogleSignIn.getClient(this,
				GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN);
		Intent intent = signInClient.getSignInIntent();
		startActivityForResult(intent, GOOGLE_GAMES_RC_SIGN_IN);

		GoogleSignInOptions signInOptions = GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN;
		GoogleSignInAccount account = GoogleSignIn.getLastSignedInAccount(this);
		if (GoogleSignIn.hasPermissions(account, signInOptions.getScopeArray())) {

			// Hide the sign-in button
			findViewById(R.id.btnGoogleSignIn).setVisibility(View.GONE);
			if (findViewById(R.id.btnAd) != null) {
				findViewById(R.id.btnAd).setVisibility(View.VISIBLE);
			}
		}
	}

	private void googleGamesSignOut() {
		GoogleSignInClient signInClient = GoogleSignIn.getClient(this,
				GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN);
		signInClient.signOut().addOnCompleteListener(this,
				new OnCompleteListener<Void>() {
					@Override
					public void onComplete(@NonNull Task<Void> task) {
						// at this point, the user is signed out.

						// Show the sign-in button
						findViewById(R.id.btnGoogleSignIn).setVisibility(View.VISIBLE);
						if (findViewById(R.id.btnAd) != null) {
							findViewById(R.id.btnAd).setVisibility(View.GONE);
						}
					}
				});
	}

	private void switchStatePreference(Preference pref)
	{
		View oldContentView = null, newContentView = null;
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			oldContentView = mCurrentContentView;
			newContentView = setupContentView();

			BitmapUtils.setScaledBackground(this, mBgFrame, R.drawable.bg_open);
			mImgLogo.setImageResource(R.drawable.bg_open_logo);

			mMenuBottomMarginView.setLayoutParams(new LinearLayout.LayoutParams(
					LayoutParams.MATCH_PARENT, 0, SIMON2_SUB_MENU_BOTTOM_MARGIN_WEIGHT));
			mMenuTopMarginView.setLayoutParams(new LinearLayout.LayoutParams(
					LayoutParams.MATCH_PARENT, 0, SIMON2_SUB_MENU_TOP_MARGIN_WEIGHT));
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			mSphinxView.setVisibility(View.GONE);
		}

		mMenuFrame.removeAllViews();
		View.inflate(this, R.layout.setting_container_frame, mMenuFrame);
		FrameLayout containerFrame = (FrameLayout) mMenuFrame
				.findViewById(R.id.settingContainerFrame);

		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
				|| AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			mSubTitleView.setVisibility(View.GONE);
		}

		mMenuState = MenuState.OPTIONS_LEVEL2;

		GameMenuUiHelper.createPreferenceFrame(this, getPreferenceManager(), pref, containerFrame);

		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			// Animate the new content view
			newContentView.startAnimation(AnimationUtils.loadAnimation(this,
					R.anim.activity_fade_in));
			oldContentView.startAnimation(AnimationUtils.loadAnimation(this,
					R.anim.activity_fade_out));

			setContentView(newContentView);
		}

		findViewById(R.id.btnBack).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{

				// WORKAROUND: for Simon 1 German version - make sure we use the
				// "voice only option"
				if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1)
				{
					AndroidPortAdditions.instance().germanVersionForceVoiceOnly(
							getPreferenceManager());
				}

				backMenuState();
			}
		});

		// Show prompt for graphic settings if needed
		if (!AndroidPortAdditions.instance().getGraphicSettingPromptShown()
				&& pref.getKey().equalsIgnoreCase("pref_scaler_option"))
		{
			Dialog dlg = DialogUtils.createNegativePositiveDialog(this, 0, R.string.ok,
					R.string.graphic_setting_message, null, null);
			dlg.show();

			AndroidPortAdditions.instance().setGraphicSettingPromptShown(true);
		}
	}

	private void resetGameFilesIfNeeded()
	{

		// If the language or music mode was changed, we need to reset the
		// expected language size so it will recreate the files.
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		String currentLanguage = prefs.getString("pref_language", null);
		String currentVoiceMode = prefs.getString("pref_voice_mode", null);
		String currentMusicMode = prefs.getString("pref_music_mode", null);

		if (!mPreviousLanguageSetting.equals(currentLanguage))
		{
			AndroidPortAdditions.instance().setGameFilesSize(-1);
		}

		// If the "original recording" setting is used, we also need to prepare
		// the game files
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			if (!mPreviousMusicSetting.equals(currentMusicMode)
					&& (currentMusicMode.equals("enhanced") || currentMusicMode
							.equals("original_recording")))
			{
				AndroidPortAdditions.instance().setShouldSwitchMusicFiles(true);
			}
		}

		// In Simon1: Also consider the case of using German or English language
		// and changing voice mode.
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
				&& !mPreviousVoiceSetting.equals(currentVoiceMode)
				&& (currentLanguage.equals("de") || currentLanguage.equals("en")))
		{
			AndroidPortAdditions.instance().setGameFilesSize(-1);
		}

		MyLog.d("GameMenuActivity: resetLanguageIfNeeded: previous language: "
				+ mPreviousLanguageSetting + " current: " + prefs.getString("pref_language", null)
				+ " previous voice mode " + mPreviousVoiceSetting + " current " + currentVoiceMode);
	}

	private boolean checkSettingChanges()
	{
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);

		// Remove "auto graphics" flag if graphic mode changed (for statistics)
		if (!mPreviousScalerSetting.equals(prefs.getString("pref_scaler_option", null)))
		{
			MyLog.d("GameMenuActivity: checkSettingChanges: graphic mode change, removing 'auto' flag");
			AndroidPortAdditions.instance().setAutoGraphicMode(false);
		}

		// Check for setting changes
		if (!AndroidPortAdditions.instance().getFirstUse())
		{
			if (!mPreviousMusicSetting.equals(prefs.getString("pref_music_mode", null)))
				return true;
			if (!mPreviousVoiceSetting.equals(prefs.getString("pref_voice_mode", null)))
				return true;
			if (!mPreviousLanguageSetting.equals(prefs.getString("pref_language", null)))
				return true;
			if (!mPreviousScalerSetting.equals(prefs.getString("pref_scaler_option", null)))
				return true;
			if (mPreviousMusicVolSetting != prefs.getInt("pref_music_volume", 192))
				return true;
			if (mPreviousSpeechVolSetting != prefs.getInt("pref_speech_volume", 192))
				return true;
			if (mPreviousSfxVolSetting != prefs.getInt("pref_sfx_volume", 192))
				return true;
			if (mPreviousSubtitleSpeedSetting != prefs.getInt("pref_subtitle_speed", 60))
				return true;
		}

		return false;
	}

	private void backMenuState()
	{
		if (mMenuState == MenuState.OPTIONS_LEVEL2)
		{
			switchStateOptions();
		} else if (mMenuState == MenuState.OPTIONS_LEVEL1)
		{
			resetGameFilesIfNeeded();
			if (checkSettingChanges())
			{
				showDialog(RESTART_DIALOG_ID);
			} else
			{
				switchStateMain();
			}
		} else
		{
			finish();
			ActivityAnimationUtil.makeActivityFadeTransition(GameMenuActivity.this);
		}
	}

	private void shareApp()
	{
		Intent sendIntent = new Intent();
		sendIntent.setAction(Intent.ACTION_SEND);
		String appStoreLink = getString(R.string.app_web_url);
		sendIntent.putExtra(Intent.EXTRA_TEXT, getString(R.string.share_message, appStoreLink));
		sendIntent.putExtra(Intent.EXTRA_SUBJECT, getString(R.string.appirator_app_title));
		sendIntent.setType("text/plain");
		sendIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		startActivity(Intent.createChooser(sendIntent,
				getResources().getText(R.string.share_chooser)));

		//StatisticsManager.getStatistics().reportShareClicked();
	}

	private void showOpenHouseDialog()
	{
		// Ask the user if he wants to start a game in 'open house'
		DialogUtils.showNegativePositiveActivity(this, R.string.no, R.string.yes,
				R.string.open_house_prompt, null, new DialogInterface.OnClickListener()
				{
					@Override
					public void onClick(DialogInterface dialog, int which)
					{
						GameMenuActivity.this.startOpenHouse();
					}
				}, mMidGame);

		ActivityAnimationUtil.makeActivityNullTransition(GameMenuActivity.this);
	}

	private void startOpenHouse()
	{
		if (mMidGame)
		{
			// Request the game activity to load in open house
			setResult(RESULT_REQUEST_OPEN_HOUSE, null);
		} else
		{
			// Start the game activity with open house mode
			Intent openHouseIntent = new Intent(this, PrepareGameFilesActivity.class).putExtra(
					EXTRA_MID_GAME, mMidGame);
			openHouseIntent.putExtra(ScummVMActivity.EXTRA_LOAD_OPEN_HOUSE, true);
			startActivity(openHouseIntent);
		}

		finish();
		ActivityAnimationUtil.makeActivityFadeTransition(this);
	}

	@Override
	public void onBackPressed()
	{
		// Implement the artificial menu backstack
		backMenuState();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		if (DialogUtils.callFromOnActivityResult(requestCode, resultCode, data))
		{
			return;
		}

		super.onActivityResult(requestCode, resultCode, data);

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G && checkGooglePlayServices()) {
			if (requestCode == GOOGLE_GAMES_RC_SIGN_IN) {
				GoogleSignInResult result = Auth.GoogleSignInApi.getSignInResultFromIntent(data);
				if (result.isSuccess()) {
					// The signed in account is stored in the result.
					GoogleSignInAccount signedInAccount = result.getSignInAccount();

					// Hide the sign-in button
					findViewById(R.id.btnGoogleSignIn).setVisibility(View.GONE);
					if (findViewById(R.id.btnAd) != null) {
						findViewById(R.id.btnAd).setVisibility(View.VISIBLE);
					}
				} else {
					// Show the sign-in button
					findViewById(R.id.btnGoogleSignIn).setVisibility(View.VISIBLE);
					if (findViewById(R.id.btnAd) != null) {
						findViewById(R.id.btnAd).setVisibility(View.GONE);
					}

				/*
				String message = result.getStatus().getStatusMessage();
				if (message == null || message.isEmpty()) {
					message = getString(R.string.failure_message_connection);
				}
				new AlertDialog.Builder(this).setMessage(message)
						.setNeutralButton(android.R.string.ok, null).show();
				*/
				}
			}

			GoogleSignInOptions signInOptions = GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN;
			GoogleSignInAccount account = GoogleSignIn.getLastSignedInAccount(this);
			if (!GoogleSignIn.hasPermissions(account, signInOptions.getScopeArray())) {
				// Show the sign-in button
				findViewById(R.id.btnGoogleSignIn).setVisibility(View.VISIBLE);
			}
		}
	}

	@Override
	protected Dialog onCreateDialog(int id)
	{
		if (id == RESTART_DIALOG_ID)
		{

			Dialog dlg = DialogUtils.createNegativePositiveDialog(this, 0, R.string.ok,
					R.string.restart_needed, null, new DialogInterface.OnClickListener()
					{

						public void onClick(DialogInterface arg0, int arg1)
						{

						}
					});

			dlg.setOnDismissListener(new OnDismissListener()
			{

				@Override
				public void onDismiss(DialogInterface arg0)
				{
					switchStateMain();
				}
			});

			return dlg;
		}

		return super.onCreateDialog(id);
	}

	private boolean mMidGame;

	private List<Integer> mSaveSlotIndexes;

	private FrameLayout mMenuFrame;

	private View mBgFrame;
	private ImageView mImgLogo;

	private View mCurrentContentView;

	// Used for preventing re-inflating of the main menu, due to performance
	// glitch in ad icon
	private View mMainMenuView;

	private View mSubTitleView;

	private View mSphinxView;

	private View mMenuBottomMarginView;
	private View mMenuTopMarginView;

	private View mShareBtn;
	
	private TextView mTxtMenuTitle;

	private boolean mFirstGoogleSignIn = true;

	private enum MenuState
	{
		MAIN, OPTIONS_LEVEL1, OPTIONS_LEVEL2;
	}

	private MenuState mMenuState = MenuState.MAIN;

	private String mPreviousMusicSetting;
	private String mPreviousVoiceSetting;
	private String mPreviousLanguageSetting;
	private String mPreviousScalerSetting;
	private int mPreviousMusicVolSetting;
	private int mPreviousSpeechVolSetting;
	private int mPreviousSfxVolSetting;
	private int mPreviousSubtitleSpeedSetting;

	private ImmersiveModeHelper mImmersiveModeHelper;

	private static final int RESTART_DIALOG_ID = 1;

	private static final float SIMON2_MAIN_MENU_BOTTOM_MARGIN_WEIGHT = 1.7f;
	private static final float SIMON2_SUB_MENU_BOTTOM_MARGIN_WEIGHT = 0.3f;

	private static final float SIMON2_MAIN_MENU_TOP_MARGIN_WEIGHT = 2.6f;
	private static final float SIMON2_SUB_MENU_TOP_MARGIN_WEIGHT = 1.6f;

}
