package org.iphsoft.simon1.ui;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;

import com.R;

/**
 * RelativeLayout with extended functionality.
 * 
 * Supports absolute positioning by percentage (Float values).
 * 
 * @author omergilad
 */
public class PercentageLayout extends ViewGroup {

	public class LayoutParams extends ViewGroup.LayoutParams {

		public LayoutParams(Context c, AttributeSet attrs)
		{
			super(c, attrs);

			if (attrs != null)
			{
				TypedArray arr = c.obtainStyledAttributes(attrs,
						R.styleable.PercentageLayout_LayoutParams);

				mPercentageLeft = arr.getFloat(R.styleable.PercentageLayout_LayoutParams_percentageLeft, 0);
				mPercentageTop = arr.getFloat(R.styleable.PercentageLayout_LayoutParams_percentageTop, 0);
				mPercentageWidth = arr.getFloat(R.styleable.PercentageLayout_LayoutParams_percentageWidth, 0);
				mPercentageHeight = arr.getFloat(R.styleable.PercentageLayout_LayoutParams_percentageHeight, 0);
			}
		}

		public LayoutParams(int width, int height)
		{
			super(width, height);
		}

		public LayoutParams(android.view.ViewGroup.LayoutParams source)
		{
			super(source);
		}

		public float mPercentageLeft = 0;
		public float mPercentageTop = 0;
		public float mPercentageWidth = 0;
		public float mPercentageHeight = 0;
	}

	public PercentageLayout(Context context)
	{
		super(context);
	}

	public PercentageLayout(Context context, AttributeSet attrs, int defStyle)
	{
		super(context, attrs, defStyle);
	}

	public PercentageLayout(Context context, AttributeSet attrs)
	{
		super(context, attrs);
	}

	@Override
	public ViewGroup.LayoutParams generateLayoutParams(AttributeSet attrs)
	{
		return new PercentageLayout.LayoutParams(getContext(), attrs);
	}

	@Override
	protected ViewGroup.LayoutParams generateLayoutParams(ViewGroup.LayoutParams p)
	{
		return new PercentageLayout.LayoutParams(super.generateLayoutParams(p));
	}

	@Override
	protected ViewGroup.LayoutParams generateDefaultLayoutParams()
	{
		return new PercentageLayout.LayoutParams(super.generateDefaultLayoutParams());
	}

	@Override
	protected boolean checkLayoutParams(android.view.ViewGroup.LayoutParams p)
	{
		return (p instanceof PercentageLayout.LayoutParams && super.checkLayoutParams(p));
	}

	@Override
	protected void onLayout(boolean changed, int l, int t, int r, int b)
	{
		for (int i = 0; i < this.getChildCount(); i++)
		{
			View v = getChildAt(i);
			LayoutParams params = (PercentageLayout.LayoutParams) v.getLayoutParams();

			int childLeft = (int) ((r - l) * params.mPercentageLeft);
			int childTop = (int) ((b - t) * params.mPercentageTop);
			int childRight = childLeft + (int) ((r - l) * params.mPercentageWidth);
			int childBottom = childTop + (int) ((b - t) * params.mPercentageHeight);

			v.layout(childLeft, childTop, childRight, childBottom);
		}
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec)
	{

		super.onMeasure(widthMeasureSpec, heightMeasureSpec);

		for (int i = 0; i < getChildCount(); i++)
		{
			View v = getChildAt(i);
			LayoutParams params = (PercentageLayout.LayoutParams) v.getLayoutParams();

			int wspec = MeasureSpec.makeMeasureSpec(
					(int) (getMeasuredWidth() * params.mPercentageWidth), MeasureSpec.EXACTLY);
			int hspec = MeasureSpec.makeMeasureSpec(
					(int) (getMeasuredHeight() * params.mPercentageHeight), MeasureSpec.EXACTLY);

			v.measure(wspec, hspec);
		}

	}
}
